@@ -10,9 +10,9 @@ from btpytools import tools
 
 
 def read_recipe(dirToSearch=""):
-    """ Read recipe file from current directory or a defined directory.
-        If multiple files present, reads the last one
-        Return False if no recipe present
+    """Read recipe file from current directory or a defined directory.
+    If multiple files present, reads the last one
+    Return False if no recipe present
     """
     if len(dirToSearch) == 0:
         dirToSearch = os.getcwd()
@@ -33,30 +33,27 @@ def read_recipe(dirToSearch=""):
 
 
 def sample_id(dirToSearch=""):
-    """return sample ID as a string
-    """
-    thisRecipe = read_recipe(dirToSearch)
-    if thisRecipe == False:
+    """return sample ID as a string"""
+    this_recipe = read_recipe(dirToSearch)
+    if not this_recipe:
         return False
     else:
-        return thisRecipe["sample"]["ID"]
+        return this_recipe["sample"]["ID"]
 
 
 def microscope(dirToSearch=""):
-    """return microscope name as a string
-    """
-    thisRecipe = read_recipe(dirToSearch)
-    if thisRecipe == False:
+    """return microscope name as a string"""
+    this_recipe = read_recipe(dirToSearch)
+    if not this_recipe:
         return False
     else:
-        return thisRecipe["SYSTEM"]["ID"]
+        return this_recipe["SYSTEM"]["ID"]
 
 
 def acq_start_time(dirToSearch=""):
-    """return acquisition start time as a string
-    """
-    thisRecipe = read_recipe(dirToSearch)
-    if thisRecipe == False:
+    """return acquisition start time as a string"""
+    this_recipe = read_recipe(dirToSearch)
+    if not this_recipe:
         return False
     else:
-        return thisRecipe["Acquisition"]["acqStartTime"]
+        return this_recipe["Acquisition"]["acqStartTime"]
